package sharecode.asynchronous.job;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.ecf.core.util.ECFException;
import org.eclipse.ecf.datashare.IChannel;
import org.eclipse.ecf.datashare.IChannelListener;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.progress.IProgressConstants;

import sharecode.core.Activator;
import sharecode.listeners.PresenceChannelListener;

public class ConnectJob extends Job {
    

    public ConnectJob(String name) {
       super(name);
     
    }
    protected IStatus run(IProgressMonitor monitor) {
       // Take care of connection
    	IChannelListener clistener = new PresenceChannelListener();
    	
    	try{	
    		
    		//TODO mesmo com o ECF desligado ele marca que conetou
    		Activator.getDefault().intializePresenceSession(clistener);
    		
    		setProperty(IProgressConstants.ICON_PROPERTY, Activator.getDefault().getImageDescriptor("icons/iconeTransp.gif").createImage());

    		if (monitor.isCanceled()) return Status.CANCEL_STATUS;
           
    		if (isModal(this)) {
                // The progress dialog is still open so
                // just open the message
                
                showResultsOk();
                
             } else {
             setProperty(IProgressConstants.KEEP_PROPERTY, Boolean.TRUE);
             setProperty(IProgressConstants.ACTION_PROPERTY, getConnectionAction());
             }
    		
            return Status.OK_STATUS;
         } catch (ECFException e) {
			// TODO Tratar LOG
        	Activator.getDefault().getLog().log(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 0, e.getLocalizedMessage(), e));
        	showResultsNotOk();
        	return Status.CANCEL_STATUS;
		} 
    }
    
    public boolean isModal(Job job) {
        Boolean isModal = (Boolean)job.getProperty(IProgressConstants.PROPERTY_IN_DIALOG);
        if(isModal == null) return false;
        return isModal.booleanValue();
     }

    protected static void showResultsOk() {
    	Activator.getDefault().getWorkbench().getDisplay().getDefault().asyncExec(new Runnable() {
           public void run() {
        	   getConnectionAction().run();
           }
        });
     }
    protected static void showResultsNotOk() {
    	Activator.getDefault().getWorkbench().getDisplay().getDefault().asyncExec(new Runnable() {
           public void run() {
        	   getNotConnectionAction().run();
           }
        });
     }

    
	protected static Action getConnectionAction() {
		return new Action("View connection status") {
			public void run() {
				MessageDialog.openInformation(Activator.getDefault().getWorkbench().getDisplay().getActiveShell(), 
                  "Connection Complete", 
                  "Your connection has been completed");
				Activator.getDefault().setConnected(true);
				startConsumer();
			}
		};
	}

	protected static Action getNotConnectionAction() {
		return new Action("View connection status") {
			public void run() {
				MessageDialog.openInformation(Activator.getDefault().getWorkbench().getDisplay().getActiveShell(), 
                  "Connection Not Complete", 
                  "Your connection has not been completed");
				Activator.getDefault().setConnected(false);
				
			}
		};
	}
	
	
	private static void startConsumer() {
		TimerConsumerJob consume = new TimerConsumerJob("Shared Code Consumer Process");
		consume.setPriority(Job.SHORT);
		consume.schedule();  // start as soon as possible
		
	}


 }
