package sharecode.ui.preferences;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferencePage;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.ControlListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Text;

import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import sharecode.core.Activator;
import sharecode.core.SCMessages;
import sharecode.core.model.preferences.PreferenceConstants;
import sharecode.core.model.preferences.Preferences;
import sharecode.util.ProjectListElement;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.net.Inet4Address;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;


/**
 * This class represents a preference page that is contributed to the
 * Preferences dialog. By subclassing <samp>FieldEditorPreferencePage</samp>,
 * we can use the field support built into JFace that allows us to create a page
 * that is small and knows how to save, restore and apply itself.
 * <p>
 * This page is used to modify preferences only. They are stored in the
 * preference store that belongs to the main plug-in class. That way,
 * preferences can be accessed directly via the preference store.
 */
public class ShareCodePreferencePage extends PreferencePage implements IWorkbenchPreferencePage {
    
    Composite content;
    Composite selectionBar;
    Composite body;
    int selectedDriverIndex;
    ArrayList definitions;
    HashMap selectedControls;
    
    Table projectTable;
	public Text channelGroup;
	public Text name;
	public Text containerType;
	public Text server;

    public void createControl(Composite parent) {
        super.createControl(parent);
    }

    protected Composite createContainer(Composite parent) {
        Composite container = new Composite(parent, SWT.NONE);
        GridLayout layout = new GridLayout();
        layout.numColumns = 1;
        layout.makeColumnsEqualWidth = false;
        container.setLayout(layout);

        GridData gridData = new GridData(GridData.FILL_BOTH |
                GridData.GRAB_VERTICAL | GridData.GRAB_HORIZONTAL);

        return container;
    }

    protected Control createContents(Composite parent) {
        this.content = createContainer(parent);
        createBody();

        return content;
    }

    private void createBody() {
        content.layout(true);
        body = new Composite(content, SWT.NONE);

        GridLayout layout = new GridLayout();
        layout.numColumns = 1;
        layout.makeColumnsEqualWidth = false;
        body.setLayout(layout);

        GridData gridData = new GridData(GridData.FILL_BOTH |
                GridData.GRAB_VERTICAL);
        body.setLayoutData(gridData);

        TabFolder tabFolder = new TabFolder(body, SWT.NONE);
        tabFolder.setLayoutData(new GridData(GridData.FILL_BOTH |
                GridData.GRAB_VERTICAL));

        Composite generalTab = createTab(SCMessages.getString("sc.tab.general"),
                SCMessages.getString("sc.tab.defineGenInfo"), tabFolder);

        GeneralPanel generalPanel = new GeneralPanel();
        generalPanel.createGeneralPanel(SCMessages.getString(
                "sc.tab.defineGenInfo"), generalTab);

        Composite shareTab = createTab(SCMessages.getString("sc.tab.sharedProjects"), SCMessages.getString("sc.tab.setSharedProjectes"), tabFolder);
        ProjectsPanel projectPanel = new ProjectsPanel();
        projectPanel.createContents(SCMessages.getString("sc.tab.selectProjects"), shareTab, ProjectListElement.stringsToProjectsList());

        Composite propertiesTab = createTab(SCMessages.getString(
                    "sc.tab.preferences"),
                SCMessages.getString("sc.tab.commonSharedPref"), tabFolder);

        PreferencesPanel preferncePanel = new PreferencesPanel();
        preferncePanel.createPreferenceDefinitionPanel(SCMessages.getString(
                "sc.tab.commonSharedPref"), propertiesTab);

        tabFolder.setSelection(0);
        body.layout(true);
        body.setRedraw(true);
        content.redraw();
    }

    private Composite createTab(String tabDesc, String toolTip,
        TabFolder tabFolder) {
        TabItem containerTab = new TabItem(tabFolder, SWT.NONE);
        containerTab.setText(tabDesc);
        containerTab.setToolTipText(toolTip);

        ScrolledComposite scroll = new ScrolledComposite(tabFolder,
                SWT.H_SCROLL | SWT.V_SCROLL | SWT.SHADOW_ETCHED_IN);

        scroll.setLayout(new GridLayout(1, true));
        scroll.setLayoutData(new GridData(GridData.FILL_BOTH |
                GridData.GRAB_VERTICAL | GridData.GRAB_HORIZONTAL));

        Composite composite = new Composite(scroll, SWT.NONE);
        scroll.setContent(composite);

        GridLayout gridLayout = new GridLayout(1, false);
        gridLayout.marginHeight = 0;
        gridLayout.marginWidth = 0;
        composite.setLayout(gridLayout);
        composite.setLayoutData(new GridData(GridData.FILL_BOTH |
                GridData.GRAB_VERTICAL | GridData.GRAB_HORIZONTAL));
        containerTab.setControl(scroll);
        containerTab.setToolTipText(toolTip);

        return composite;
    }

    public void init(IWorkbench workbench) {
    }

    public boolean performOk() {
        return true;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.eclipse.jface.preference.PreferencePage#performDefaults()
     */
    protected void performDefaults() {
        super.performDefaults();
        
        body.dispose();
        createBody();
        content.layout(true);
        body.setRedraw(true);
        content.redraw();
        
        Preferences preferences = Activator.getDefault().getPreferences();
    	
    	preferences.setChannelId(null);
    	preferences.setContainerType(null);
    	preferences.setLocalName(null);
    	preferences.setTargetServer(null);
           

    }


    
    
    /*
     * 
     * 
     */
    protected void performApply(){
    	
    	Preferences preferences = Activator.getDefault().getPreferences();
    	
    	preferences.setChannelId(channelGroup.getText());
    	preferences.setContainerType(containerType.getText());
    	preferences.setLocalName(name.getText());
    	preferences.setTargetServer(server.getText());
    	    	
    	Iterator iter = ProjectListElement.stringsToProjectsList().iterator(); 
    		
    		
    	
    	while (iter.hasNext()) {
			ProjectListElement element = (ProjectListElement) iter.next();
			IProject project = element.getProject();
			//TODO isso precisa ser configurado
			IFolder folder = project.getFolder("/.sharedcode");

			if(!folder.exists()){
				try{
					folder.create(true, true, null);
				}
				catch (CoreException e){
					e.printStackTrace();
				}
			}
			
			final IFile f = folder.getFile("sharedcode.properties");

			try {
				if(!f.exists()){
					
					String license = "license="+((Combo)selectedControls.get(element)).getText();
					ByteArrayInputStream is = new ByteArrayInputStream(license.getBytes());
					f.create(is, true, null);
					
				}
				
			} catch (CoreException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}


			
			System.out.println(element.getID() +" :: "+ ((Combo)selectedControls.get(element)).getText());
			
		}
		
    }
    /*
     * (non-Javadoc)
     *
     * @see org.eclipse.jface.preference.IPreferencePage#performCancel()
     */
    public boolean performCancel() {
        return true;
    }

    /*
     * @see SelectionListener#widgetSelected(SelectionEvent)
     */
    public void widgetSelected(SelectionEvent e) {
        selectedDriverIndex = ((Combo) e.getSource()).getSelectionIndex();
        
        System.out.println(" Indice de combo  "+selectedDriverIndex);
        
        body.dispose();
        createBody();
        content.layout(true);
        body.setRedraw(true);
        content.redraw();
        body.redraw();
        content.update();
    }

    public void widgetDefaultSelected(SelectionEvent e) {
        widgetSelected(e);
    }

    class ProjectsPanel {
        protected Control createContents(String title, Composite parent,
            List list) {
            return createProjectDefinitionPanel(title, parent, list);
        }

        protected Control createProjectDefinitionPanel(String title,
            Composite composite, List list) {
            Group defPanel = new Group(composite, SWT.NONE);
            defPanel.setText(title);

            GridLayout layout = new GridLayout(3, false);
            defPanel.setLayout(layout);

            GridData gridData = new GridData(); 

            defPanel.setLayoutData(gridData);

            Iterator properties = list.iterator();
            selectedControls = new HashMap();

            while (properties.hasNext()) {
                ProjectListElement item = (ProjectListElement) properties.next();
                Control control = null;
                control = createComboLicenseOnPanel("", item.getID(), defPanel);
                selectedControls.put(item, control);
            }

            composite.setSize(composite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
            composite.layout();

            return defPanel;
        }

        protected Text createLabeledText(String title, String value,
            boolean modifiable, Composite defPanel) {
            GridData gridData;
            Label label = new Label(defPanel, SWT.WRAP);
            gridData = new GridData();
            label.setLayoutData(gridData);
            label.setText(title);

            Text fText;

            if (modifiable) {
                fText = new Text(defPanel, SWT.SHADOW_IN | SWT.BORDER);
            } else {
                fText = new Text(defPanel,
                        SWT.SHADOW_IN | SWT.BORDER | SWT.READ_ONLY);
            }

            gridData = new GridData(GridData.FILL_HORIZONTAL |
                    GridData.GRAB_HORIZONTAL);
            gridData.horizontalSpan = 2;
            fText.setLayoutData(gridData);
            fText.setText(value);

            return fText;
        }

        protected Button createLabeledCheck(String title, boolean value,
            Composite defPanel) {
            GridData gridData;
            Label label = new Label(defPanel, SWT.WRAP);
            gridData = new GridData();
            label.setLayoutData(gridData);
            label.setText(title);

            Button fButton = new Button(defPanel, SWT.CHECK);
            gridData = new GridData(GridData.FILL_HORIZONTAL |
                    GridData.GRAB_HORIZONTAL);
            gridData.horizontalSpan = 2;
            fButton.setLayoutData(gridData);
            fButton.setSelection(value);

            return fButton;
        }

        private Combo createComboLicense(Composite defPanel) {
            Combo combo2 = new Combo(defPanel, SWT.BORDER);

            // add the value catch in the properties and adds in comboBox
            int comboLength = Integer.parseInt(SCMessages.getString(
                        "sc.cmd.lenght.licenses"));
            String combo = "sc.cmb.licenses.";

            for (int i = 0; i < comboLength; i++) {
                combo2.add(SCMessages.getString(combo + i));
            }
            combo2.select(0);
            
            

            return combo2;
        }

        protected Combo createComboLicenseOnPanel(String title, String value,
            Composite defPanel) {
            GridData gridData;
            Label label = new Label(defPanel, SWT.WRAP);
            gridData = new GridData();
            label.setLayoutData(gridData);
            label.setText(title + value);

            final Combo license = createComboLicense(defPanel);
            gridData = new GridData(); 
            gridData.horizontalSpan = 1;

            license.setLayoutData(gridData);

            Button fButton = new Button(defPanel, SWT.CHECK);
            fButton.setSelection(true);
            fButton.setLayoutData(new GridData());
            return license;
        }

        protected Text createLabeledFile(String title, String value,
            Composite defPanel) {
            GridData gridData;
            Label label = new Label(defPanel, SWT.WRAP);
            gridData = new GridData();
            label.setLayoutData(gridData);
            label.setText(title);

            final Text fText = new Text(defPanel, SWT.SHADOW_IN | SWT.BORDER);
            gridData = new GridData(GridData.FILL_HORIZONTAL |
                    GridData.GRAB_HORIZONTAL);
            gridData.horizontalSpan = 1;
            fText.setLayoutData(gridData);
            fText.setText(value);

            Button fButton = new Button(defPanel, SWT.PUSH);
            fButton.setText("...");
            fButton.setLayoutData(new GridData());
            fButton.addSelectionListener(new SelectionListener() {
                    public void widgetSelected(SelectionEvent e) {
                        FileDialog dlg = new FileDialog(getControl().getShell());
                        dlg.setFileName(fText.getText());

                        String res = dlg.open();

                        if (res != null) {
                            fText.setText(res);
                        }
                    }

                    public void widgetDefaultSelected(SelectionEvent e) {
                        widgetSelected(e);
                    }
                });

            return fText;
        }
    }

    class GeneralPanel {

		protected Control createGeneralPanel(String title, Composite composite) {
            Group defPanel = new Group(composite, SWT.NONE);
            defPanel.setText(title);

            GridLayout layout = new GridLayout(2, false);
            defPanel.setLayout(layout);

            GridData gridData = new GridData();

            defPanel.setLayoutData(gridData);
            
            Preferences preferences = Activator.getDefault().getPreferences();
            
            containerType = createLabeledText(SCMessages.getString("sc.txt.containerType"),preferences.getContainerType(), defPanel);
            server = createLabeledText(SCMessages.getString("sc.txt.ecfServerURL"), preferences.getTargetServer(),defPanel);
            channelGroup = createLabeledText(SCMessages.getString("sc.txt.channelGroupName"),preferences.getChannelId(),defPanel);
            
            name = createLabeledText(SCMessages.getString("sc.txt.yourName"), preferences.getLocalName(),defPanel);
            createEnableButton(SCMessages.getString("sc.ckb.enable"), defPanel);

            composite.setSize(composite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
            composite.layout();

            return defPanel;
        }

        protected Text createLabeledText(String value, String text, Composite defPanel) {
            GridData gridData;
            Label label = new Label(defPanel, SWT.WRAP);
            gridData = new GridData();
            // gridData.horizontalSpan = 1;
            label.setLayoutData(gridData);
            label.setText(value);

            final Text fText = new Text(defPanel, SWT.SHADOW_IN | SWT.BORDER);
            gridData = new GridData(); 

            gridData.widthHint = 200;
            fText.setLayoutData(gridData);
            fText.setText(text);
            return fText;
        }

        private void createEnableButton(String title, Composite defPanel) {
            Button fButton = new Button(defPanel, SWT.CHECK);
            fButton.setText(title);
            fButton.setLayoutData(new GridData());

            fButton.addSelectionListener(new SelectionListener() {
                    public void widgetSelected(SelectionEvent e) {
                    }

                    public void widgetDefaultSelected(SelectionEvent e) {
                        widgetSelected(e);
                    }
                });
        }
    }

    class PreferencesPanel {
        protected Control createPreferenceDefinitionPanel(String title,
            Composite composite) {
            Group defPanel = new Group(composite, SWT.NONE);
            defPanel.setText(title);

            GridLayout layout = new GridLayout(1, false);
            GridData gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
            defPanel.setLayout(layout);
            defPanel.setLayoutData(gridData);

            Group numberSearch = new Group(defPanel, SWT.BORDER);
            numberSearch.setText(SCMessages.getString(
                    "sc.txt.searchResultsGroupModel"));
            numberSearch.setLayout(new GridLayout(2, false));
            numberSearch.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
            createLabeledText(SCMessages.getString("sc.txt.resultsDisplay"),
                SCMessages.getString("sc.constant.ten"), true, numberSearch);

            Group border = new Group(defPanel, SWT.BORDER);
            border.setText(SCMessages.getString(
                    "sc.txt.searchResultsGroupModel"));
            border.setLayout(new GridLayout());
            border.setLayoutData(gridData);
            createLabeledRadio(SCMessages.getString("sc.rdb.searchSCP"), false,
                border);
            createLabeledRadio(SCMessages.getString("sc.rdb.licenses"), false,
                border);
            createLabeledRadio(SCMessages.getString("sc.rdb.noGrouping"),
                true, border);

            Group borderTypeSearch = new Group(defPanel, SWT.BORDER);
            borderTypeSearch.setText(SCMessages.getString("sc.txt.typeSearch"));
            borderTypeSearch.setLayout(new GridLayout());
            borderTypeSearch.setLayoutData(new GridData(
                    GridData.FILL_HORIZONTAL));
            createLabeledRadio(SCMessages.getString("sc.rdb.javaSourceFiles"),
                true, borderTypeSearch);

            composite.setSize(composite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
            composite.layout();

            return defPanel;
        }

        protected Text createLabeledText(String title, String value,
            boolean modifiable, Composite defPanel) {
            GridData gridData;
            Label label = new Label(defPanel, SWT.WRAP);
            gridData = new GridData(GridData.FILL_BOTH);
            label.setLayoutData(gridData);
            label.setText(title);

            Text fText;

            if (modifiable) {
                fText = new Text(defPanel, SWT.SHADOW_IN | SWT.BORDER);
            } else {
                fText = new Text(defPanel,
                        SWT.SHADOW_IN | SWT.BORDER | SWT.READ_ONLY);
            }

            gridData = new GridData(GridData.FILL_HORIZONTAL |
                    GridData.GRAB_HORIZONTAL);
            // gridData.horizontalSpan = 2;
            gridData.widthHint = 35;
            fText.setLayoutData(gridData);
            fText.setText(value);

            return fText;
        }

        protected Button createLabeledRadio(String title, boolean value,
            Composite defPanel) {
            GridData gridData;
            gridData = new GridData();

            Button fButton = new Button(defPanel, SWT.RADIO);
            gridData = new GridData();
            fButton.setLayoutData(gridData);
            fButton.setSelection(value);
            fButton.setText(title);

            return fButton;
        }
    }
}

